from django.conf.urls import url

from . import views

urlpatterns = [
    url('^$', views.styleguide, name='styleguide', kwargs={'page': 'index'}),
    url('^(?P<page>.*)/$', views.styleguide, name='styleguide'),
]



import datetime
import json
import os
import random
import urllib.request
from multiprocessing.pool import ThreadPool

import requests
from django.conf import settings
from django.contrib.auth.models import User
from django.core.files.images import ImageFile
from django.core.serializers.json import DjangoJSONEncoder
from django.utils import lorem_ipsum
from django.utils.text import capfirst
from taggit.models import Tag
from wagtail.wagtailimages.models import Image

n = random.randint


def rbool(prob=0.5):
    return random.random() < prob


def lpar(num):
    """``num`` random paragraphs of lorem ipsum."""
    return ''.join('<p>{0}</p>'.format(p)
                   for p in lorem_ipsum.paragraphs(num, False))


def lwords(num):
    """``num`` random lorem ipsum words."""
    return capfirst(lorem_ipsum.words(num, False))


def tel():
    """ random international format (aus) telephone number eg. +61 (0) 412 345 678"""
    return '+{0} ({1}) {2} {3} {4}'.format(
        n(10, 99), n(0, 9), n(100, 999), n(100, 999), n(100, 999))


def rdate(future=True):
    """
    A random date. Set ``future`` to True for dates in the future, False for
    dates in the past. Possible dates span up to five years from today.
    """
    now = datetime.date.today()
    delta = datetime.timedelta(years=n(0, 4), days=n(0, 365))
    if future:
        return now + delta
    else:
        return now - delta


def r_heading(level, num):
    return '<h{0}>{1}</h{0}>'.format(level, lwords(num))


def r_bold(num):
    return '<p><b>{0}</b></p>'.format(lwords(num))


def r_italic(num):
    return '<p><i>{0}</i></p>'.format(lwords(num))


def r_ordered_list(list_elements):
    list_elems = []
    for i in range(list_elements):
        list_elems.append('<li>{0}</li>\n'.format(lwords(n(2, 4))))
    return '<ol>\n{0}</ol>'.format('\n'.join(map(str, list_elems)))


def r_unordered_list(list_elements):
    list_elems = []
    for i in range(list_elements):
        list_elems.append('<li>{0}</li>\n'.format(lwords(n(2, 4))))
    return '<ul>\n{0}</ul>'.format('\n'.join(map(str, list_elems)))


def hr():
    return '<hr>'


def r_anchor(num):
    return("<a href='http://example.com'>{0}</a>".format(lwords(num)))


def r_image():
    image = get_random_image()
    return('<embed alt="Example image" embedtype="image" format="fullwidth" id="{0}"/>'.format(image.id))


def embed_video():
    return '<embed embedtype="media" url="{0}"/>'.format(get_random_video_url())


def rich_text_example():
    """Layout all possible elements that can be used in rich text fields."""
    return open(os.path.join(os.path.dirname(__file__), 'rich_text_example.html'), 'r').read()


def random_rich_text_example(num, embeds=False):
    """
    Layout some possible elements that can be used in rich text fields in a random fashion
    Takes one argument, the number of elements. Set embeds=True if you wish to include images
    and videos
    """
    funcs = [
        lambda: r_heading(n(2, 5), n(1, 3)),
        lambda: r_bold(n(3, 5)),
        lambda: r_italic(n(3, 5)),
        lambda: r_ordered_list(n(3, 5)),
        lambda: r_unordered_list(n(3, 5)),
        lambda: r_anchor(n(3, 5)),
        lambda: lpar(n(3, 5)),
        hr,
    ]
    if embeds:
        funcs.extend([r_image, embed_video])
    return '\n'.join([random.choice(funcs)() for _ in range(num)])


def json_dumps(x):
    """Dump a value to JSON using the Django JSON encoder"""
    return json.dumps(x, cls=DjangoJSONEncoder)


def shuf(l):
    """Generate a shuffled copy of a list"""
    l = l[:]
    random.shuffle(l)
    return l


class once(object):
    """A decorator that only calls the decorated function once"""
    has_run = False
    value = None
    fn = None

    def __init__(self, fn):
        self.fn = fn

    def __call__(self, *args, **kwargs):
        if self.has_run:
            return self.value

        self.value = self.fn(*args, **kwargs)
        self.has_run = True
        return self.value


flickr_api_key = 'f6b85533d3860015082acff890dce7cf'
flickr_endpoint = 'https://api.flickr.com/services/rest/'
flickr_photo_url = 'https://farm{farm}.staticflickr.com/{server}/{id}_{secret}_k.jpg'


def download_image_file(image_url, image_path):
    print("Downloading", image_url)
    image_stream = urllib.request.urlopen(image_url)
    tmp_path = image_path + '.part'
    with open(tmp_path, "wb") as out:
        out.write(image_stream.read())
    os.rename(tmp_path, image_path)


@once
def download_random_images(image_categories=None):
    """Download a bunch of random images from the Flickr public API"""
    if image_categories is None:
        image_categories = [('cats', 10), ('dogs', 10), ('nature', 10),
                            ('pattern', 10), ('architecture', 10)]

    image_prefix = 'original_images/test-images'
    image_dirname = os.path.join(settings.MEDIA_ROOT, image_prefix)

    os.makedirs(image_dirname, exist_ok=True)

    def job(category, num, photo):
        filename = "{0}-{1}.jpg".format(category, num)
        image_path = os.path.join(image_dirname, filename)

        if not os.path.exists(image_path):
            response = requests.get(flickr_endpoint, params={
                'method': 'flickr.photos.getSizes',
                'api_key': flickr_api_key,
                'photo_id': photo['id'],
                'format': 'json', 'nojsoncallback': '1'})
            sizes = sorted(response.json()['sizes']['size'], reverse=True,
                           key=lambda p: int(p['width']) * int(p['height']))
            largest = sizes[0]
            image_url = largest['source']
            download_image_file(image_url, image_path)

        image = Image(
            title=photo['title'] or '{} {}'.format(category, num),
            file=ImageFile(open(image_path, 'rb'), name=filename))
        return (image, category)

    jobs = []
    for category, count in image_categories:
        json_path = os.path.join(image_dirname, '{}.json'.format(category))
        # Annoying race condition when implicitly creating tags by assigning
        # them to objects. Creating them straight up should sort that out.
        Tag.objects.get_or_create(name=category)
        if not os.path.exists(json_path):
            response = requests.get(flickr_endpoint, params={
                'method': 'flickr.photos.search',
                'api_key': flickr_api_key,
                'tags': category,
                'media': 'photos', 'per_page': count,
                'format': 'json', 'nojsoncallback': '1'})
            with open(json_path, 'w') as json_file:
                json_file.write(response.text)
            json_data = response.json()
        else:
            with open(json_path, 'r') as json_file:
                json_data = json.load(json_file)

        for num, photo in enumerate(json_data['photos']['photo']):
            jobs.append((category, num, photo))

    with ThreadPool(4) as pool:
        async_result = pool.starmap_async(job, jobs)
        pool.close()
        results = async_result.get(240)
        for image, category in results:
            image.save()
            image.tags.add(category)
        pool.join()


def get_random_image(category=None):
    download_random_images()
    images = Image.objects.order_by('?')
    if category:
        images = images.filter(tags__name__in=[category])
    return images.first()


def get_random_video_url():
    return random.choice([
        'https://youtu.be/H9VVkwRb_7M',
        'https://youtu.be/2WWwkArWajM',
        'https://youtu.be/yYaJWO_mNWs',
        'https://youtu.be/1kThvYlo2Lc',
        'https://youtu.be/Ji9qSuQapFY',
        'https://vimeo.com/155536745',
        'https://vimeo.com/179936131',
        'https://vimeo.com/188197250',
        'https://vimeo.com/175535505',
        'https://vimeo.com/188321226'
    ])


def make_superuser():
    User.objects.create_superuser(username='admin', email='', password='p')



from django.conf import settings
from django.template.loader import render_to_string
from django.utils.html import mark_safe
from wagtail.wagtailcore import hooks

from .models import AnalyticsSettings


class AnalyticsWarningPanel(object):
    name = 'analytics_warning_panel'
    order = 200

    def __init__(self, request):
        if not request.site:
            raise ValueError("Need an active site to show analytics warnings")
        self.request = request

    def render(self):
        instance = AnalyticsSettings.for_site(self.request.site)
        if not instance.google_analytics:
            return mark_safe(render_to_string('coop/analytics_warning.html', {
                'request': self.request,
            }))
        else:
            return ""


@hooks.register('construct_homepage_panels')
def analytics_warnings(request, panels):
    if settings.DEBUG:
        return
    try:
        analytics_panels = AnalyticsWarningPanel(request)
        panels.append(analytics_panels)
    except ValueError:
        pass



import posixpath

# Misc rubbish
SITE_ID = 1
DEFAULT_PER_PAGE = 20

# Sites built on coop all share a common URL structure and WSGI application
ROOT_URLCONF = 'coop.urls'
WSGI_APPLICATION = 'coop.wsgi.application'


# URLs for assets
ASSETS_URL = '/assets/'
MEDIA_URL = posixpath.join(ASSETS_URL, 'media/')
STATIC_URL = posixpath.join(ASSETS_URL, 'static/')


# What is installed by default
INSTALLED_APPS = [
    'coop',

    # Third party apps
    'taggit',
    'modelcluster',
    'wagtailmetadata',
    'wagtailaccessibility',
    'wagtailfontawesome',

    # Django apps
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django.contrib.admin',

    # Wagtail apps
    'wagtail.wagtailcore',
    'wagtail.wagtailadmin',
    'wagtail.wagtaildocs',
    'wagtail.wagtailsnippets',
    'wagtail.wagtailsites',
    'wagtail.wagtailusers',
    'wagtail.wagtailimages',
    'wagtail.wagtailembeds',
    'wagtail.wagtailsearch',
    'wagtail.wagtailredirects',
    'wagtail.contrib.settings',
    'wagtail.contrib.table_block',
    'wagtail.contrib.wagtailsitemaps',
]


# Localisation - most sites are Australian
TIME_ZONE = 'Australia/Hobart'
LANGUAGE_CODE = 'en-au'

USE_I18N = True
USE_L10N = False
USE_TZ = True


# Standard middleware required by Wagtail
MIDDLEWARE_CLASSES = [
    'django.middleware.common.CommonMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',

    'wagtail.wagtailcore.middleware.SiteMiddleware',
    'wagtail.wagtailredirects.middleware.RedirectMiddleware',
]


# Wagtail uses Django templates, but we use Jinja templates.
# Some common Jinja extensions are used that cover most use-cases we need.
TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.contrib.auth.context_processors.auth',
                'django.template.context_processors.debug',
                'django.template.context_processors.i18n',
                'django.template.context_processors.media',
                'django.template.context_processors.static',
                'django.template.context_processors.tz',
                'django.contrib.messages.context_processors.messages',
                'django.template.context_processors.request',
                'wagtail.contrib.settings.context_processors.settings',
            ]
        }
    },
    {
        'BACKEND': 'django.template.backends.jinja2.Jinja2',
        'APP_DIRS': True,
        'OPTIONS': {
            'extensions': [
                'wagtail.wagtailcore.jinja2tags.core',
                'wagtail.wagtailadmin.jinja2tags.userbar',
                'wagtail.wagtailimages.jinja2tags.images',
                'wagtail.contrib.settings.jinja2tags.settings',
                'coop.jinja2tags.core.Extension',
                'wagtailaccessibility.jinja2tags.tota11y',
                'wagtailmetadata.jinja2tags.WagtailMetadataExtension',
            ],
        },
    }
]


# Who to email when things break
ADMINS = [
    ('Django debug', 'django@takeflight.com.au'),
]
LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'filters': {
        'require_debug_false': {
            '()': 'django.utils.log.RequireDebugFalse'
        }
    },
    'handlers': {
        'mail_admins': {
            'level': 'ERROR',
            'filters': ['require_debug_false'],
            'class': 'django.utils.log.AdminEmailHandler'
        }
    },
    'loggers': {
        'django.request': {
            'handlers': ['mail_admins'],
            'level': 'ERROR',
            'propagate': True,
        },
    }
}



import json
import re

import jinja2
import jinja2.ext
from django.conf import settings
from django.contrib.messages.context_processors import get_messages
from django.contrib.staticfiles.storage import staticfiles_storage
from django.core.serializers.json import DjangoJSONEncoder
from django.core.urlresolvers import reverse
from django.forms.utils import flatatt
from django.template.loader import render_to_string
from django.utils.html import format_html
from wagtail.wagtailcore.models import Page


def debug_variable(variable):
    """Print all the fields available on a variable"""
    return format_html('<pre>{}</pre>', dir(variable))


@jinja2.contextfunction
def csrf_field(context):
    """Print the hidden CSRF token field"""
    csrf_token = context.get('csrf_token')
    if not csrf_token or csrf_token == 'NOTPROVIDED':
        return format_html("")

    return format_html("<input{}>", flatatt({
        'type': 'hidden',
        'name': 'csrfmiddlewaretoken',
        'value': csrf_token}))


def url(name, *args, **kwargs):
    """Reverse a URL name. Alias of ``django.core.urlresolvers.reverse``"""
    return reverse(name, args=args or None, kwargs=kwargs or None)


@jinja2.contextfunction
def site_root(context):
    """Get the root page for the site"""
    # This lookup will be cached on the intermediary objects, so this will only
    # hit the DB once per request
    return context['request'].site.root_page.specific


def svg_inline(name):
    """
    Inline an SVG image. SVG images are expected to be a template, not kept in
    the static files
    """
    return render_to_string('svgs/%s.svg' % name)


@jinja2.contextfunction
def breadcrumbs(context, page=None):
    """Print the top navigation menu for this site"""
    request = context.get('request')
    root = site_root(context)
    if page is None:
        page = context.get('page')
    ancestors = page.get_ancestors().filter(depth__gte=root.depth)

    return jinja2.Markup(render_to_string('tags/breadcrumbs.html', {
        'page': page,
        'ancestors': ancestors,
        'request': request,
    }))


def js_context(key, value):
    """ Adds variables to the window for use by other scripts. """
    return format_html('window.appData["{0}"] = {1};'.format(key.upper(), json.dumps(value)))


def model_classname(model_or_instance):
    """
    Generate a CSS class name from a Page model

    Usage::

        <html class="{{ self|model_classname }}">
    """
    if isinstance(model_or_instance, Page):
        model_or_instance = model_or_instance.content_type.model_class()

    try:
        meta = model_or_instance._meta
        return 'page-{0}-{1}'.format(meta.app_label, meta.model_name)
    except AttributeError:
        return ''


@jinja2.contextfunction
def messages(context):
    """Get any messages from django.contrib.messages framework"""
    return get_messages(context.get('request'))


def json_dumps(value):
    """
    Dump the value to a JSON string. Useful when sending values to JavaScript
    """
    return jinja2.Markup(json.dumps(value, cls=DjangoJSONEncoder))


not_digit_re = re.compile(r'[^0-9+]+')


def tel(value):
    return 'tel:{}'.format(not_digit_re.sub('-', value).strip('-'))


class Extension(jinja2.ext.Extension):
    def __init__(self, environment):
        super(Extension, self).__init__(environment)

        self.environment.globals.update({
            'breadcrumbs': breadcrumbs,
            'site_root': site_root,
            'csrf_field': csrf_field,
            'static': staticfiles_storage.url,
            'url': url,
            'svg': svg_inline,
            'js_context': js_context,
            'model_classname': model_classname,
            'messages': messages,
            'DEBUG': settings.DEBUG,
        })

        self.environment.filters.update({
            'model_classname': model_classname,
            'debug_variable': debug_variable,
            'json': json_dumps,
            'tel': tel,
        })



"""
Some testing tools that coop projects can use.

Most projects will want to do something like:

.. code-block:: python

    from coop.test import TestAllTheThingsMixin
    from django.test import TestCase

    class TestAllTheThings(TestAllTheThingsMixin, TestCase):
        pass
"""
from io import StringIO
from unittest import mock
from urllib.parse import urlsplit

from django.conf import settings
from django.contrib.staticfiles import finders
from django.core.management import call_command
from wagtail.wagtailimages.tests.utils import get_test_image_file

from coop.utils import testdata


class CoopAssertionMixin():
    def assertNoMissingMigrations(self):
        try:
            out = StringIO()
            call_command(
                *'makemigrations --check --dry-run --no-color --name=missing_migration'.split(),
                stdout=out)
        except SystemExit:
            out.seek(0)
            self.fail(msg='Missing migrations:\n\n' + out.read())

    def assertTestdataWorks(self):
        """
        Run the ``testdata`` management command. The tests pass if no errors
        are thrown
        """
        def mocked_download_image(image_url, image_path):
            """
            Don't actually go and download the image from lorem pixel, as that
            is quite slow, but still make an image file in the correct place.
            """
            test_image = get_test_image_file(image_path)
            with open(image_path, 'wb') as f:
                test_image.file.seek(0)
                f.write(test_image.file.read())

        with mock.patch.object(testdata, 'download_image_file', mocked_download_image):
            call_command('testdata', stdout=StringIO())
            call_command('update_index', stdout=StringIO())

    def assertStandardFilesExist(self):
        for standard_file in ['/favicon.ico', '/robots.txt', '/humans.txt']:
            response = self.client.get(standard_file, follow=True)
            final_url = response.redirect_chain[-1][0]
            path = urlsplit(final_url).path
            self.assertTrue(path.startswith(settings.STATIC_URL))
            tail = path[len(settings.STATIC_URL):]
            self.assertTrue(finders.find(tail), msg='{} exists'.format(path))

    def assert404PageRenders(self):
        url = '/quite/likely/not/a/page/'
        response = self.client.get(url)
        self.assertContains(response, url, status_code=404)

    def assert500PageRenders(self):
        response = self.client.get('/500/')
        self.assertEqual(response.status_code, 500)


class TestAllTheThingsMixin(CoopAssertionMixin):
    """
    Run all the stand-alone assertions that check for a coop project in a good
    state.
    """
    def test_migrations(self):
        self.assertNoMissingMigrations()

    def test_testdata(self):
        self.assertTestdataWorks()

    def test_standard_files_exist(self):
        self.assertStandardFilesExist()

    def test_error_pages(self):
        self.assert404PageRenders()
        self.assert500PageRenders()



"""
Remove the automatically generated site and home page
"""

from django.db import migrations


def remove_site(apps, schema):
    Site = apps.get_model('wagtailcore', 'site')
    Page = apps.get_model('wagtailcore', 'page')

    # Remove all the pre-defined sites
    Site.objects.all().delete()

    # Delete the home page
    Page.objects.get(pk=2).delete()
    # Fix the root page to show it has no children
    Page.objects.filter(pk=1).update(numchild=0)


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailcore', '__latest__'),
    ]

    operations = [
        migrations.RunPython(remove_site, migrations.RunPython.noop)
    ]



#!/usr/bin/env python3
"""Install coop"""

from setuptools import find_packages, setup

with open('coop/_version.py', 'r') as f:
    version = None
    exec(f.read())

with open('README.rst', 'r') as f:
    readme = f.read()


install_requires = ['wagtail>=1.0']

setup(
    name='coop',
    version=version,
    description='Standard base to build Nestbox sites from',
    long_description=readme,
    author='Tim Heap',
    author_email='tim@takeflight.com.au',
    url='https://gitlab.com/takeflight/coop',

    install_requires=[
        'psycopg2==2.6.1',
        'wagtail~=1.9.0',
        'django>=1.10,<1.11',
        'pytz>=0',
        'Jinja2==2.8',
        'wagtail-metadata~=0.3.1',
        'wagtail-accessibility~=0.1.1',
        'wagtailfontawesome~=1.0',
        'requests>=2.10.0,<3',
        'elasticsearch~=1.9.0',
    ],

    zip_safe=False,
    license='BSD License',

    packages=find_packages(exclude=['tests*']),

    include_package_data=True,
    package_data={},

    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Framework :: Django',
        'License :: OSI Approved :: BSD License',
    ],
)



from django.db import models
from django.utils.translation import ugettext_lazy
from wagtail.wagtailadmin.edit_handlers import FieldPanel, MultiFieldPanel
from wagtail.wagtailcore import models as wagtailmodels
from wagtail.wagtailimages.edit_handlers import ImageChooserPanel
from wagtailmetadata.models import MetadataPageMixin


class Page(MetadataPageMixin, wagtailmodels.Page):
    """
    A base page with social metadata and a more sensible default template.
    """

    # Disable the creation of the related field accessor on Page
    # This prevents conflicts with model names and field names
    page_ptr = models.OneToOneField(wagtailmodels.Page, parent_link=True,
                                    related_name='+')

    # We override promote panels to remove show_in_menus
    promote_panels = [
        MultiFieldPanel([
            FieldPanel('slug'),
            FieldPanel('seo_title'),
            FieldPanel('search_description'),
            ImageChooserPanel('search_image'),  # From MetadataPageMixin
        ], ugettext_lazy('Common page configuration')),
    ]

    def get_template(self, request, *args, **kwargs):
        """
        Templates are named like ``layouts/app/model.html``.
        """
        return "layouts/{}/{}.html".format(
            self._meta.app_label, self._meta.model_name)

    class Meta:
        abstract = True



from django.db import models
from wagtail.contrib.settings.models import BaseSetting, register_setting


@register_setting(icon='site')
class AnalyticsSettings(BaseSetting):
    google_analytics = models.CharField(
        max_length=64, blank=True, null=True,
        help_text='Your google analytics code')



"""
Lazily proxy a view module, preventing circular imports when used in a
models file. Useful to work with Wagtail Pages, where the view must be
defined on the model.
"""

from django.utils.functional import cached_property
from django.utils.module_loading import import_module


class ModelViewProxy(object):
    """
    Lazily proxy a view module, preventing circular imports when used in a
    models file. Useful to work with Wagtail Pages, where the view must be
    defined on the model.

    Use like:

    .. code-block:: python

        # myapp/models.py
        views = ModelViewProxy('myapp.views')

        class MyPage(Page):
            serve = views.mypage_detail

    .. code-block:: python

        # myapp/views.py
        def mypage_detail(request, mypage):
            ...

    The ``myapp.views.mypage_detail`` view will be passed arguments ``request``
    and ``page``, as well as any extra args or kwargs from the URL.
    """

    def __init__(self, view_path):
        self.view_path = view_path

    @cached_property
    def module(self):
        """Get the actual view module this instance is proxying"""
        return import_module(self.view_path)

    def __getattr__(self, name):
        """Get a proxy for a view from the proxied module with the same name"""
        return viewproxy(self, name).proxy()

    def __repr__(self):
        return '<ModelViewProxy "{0}">'.format(self.view_path)


class viewproxy(object):
    """Proxy a single view in a model, in a lazy manner."""

    def __init__(self, model_view_proxy, name):
        self.model_view_proxy = model_view_proxy
        self.name = name

    @cached_property
    def view(self):
        """The proxied view function, fetched lazily when required"""
        return getattr(self.model_view_proxy.module, self.name)

    def proxy(self):
        """Make a function that calls the proxied view."""
        def proxy(page, request, *args, **kwargs):
            return self.view(request, page, *args, **kwargs)
        return proxy
